%%
%%  wings__font.erl --
%%
%%     Generic font operations.
%%
%%  Copyright (c) 2005-2011 Bjorn Gustavsson.
%%
%%  See the file "license.terms" for information on usage and redistribution
%%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
%%
%%     $Id$
%%

-module(wings__font).
-export([new/6,key/1,desc/1,width/1,height/1,draw/2,char/2,char_width/2,
	bold_char/2,bold_char_width/2]).

-record(font, {key, desc, width, height, glyph_tab, bitmaps}).

new(Key, Desc, Width, Height, GlyphTab, Bitmaps) ->
    #font{key = Key, desc = Desc, width = Width, height = Height, glyph_tab = GlyphTab, bitmaps = Bitmaps}.

draw(Font, [C|T]) ->
    char(Font, C),
    draw(Font, T);
draw(_Font, []) -> ok.

key(#font{key = Key}) -> Key.
desc(#font{desc = Desc}) -> Desc.

char_width(Font, C) ->
    element(1, glyph_info(Font, C)).
width(#font{width = Width}) ->
    insert_glyph(char_width, Width),
    Width.
height(#font{height = Height}) ->
    insert_glyph(char_height, Height),
    Height.

bold_char(Font, C) ->
    Glyph = glyph_info(Font, C),
    draw_glyph(Glyph),
    Cw = glyph_width(Glyph),
    gl:bitmap(1, 1, 0, 0, -Cw+1, 0, <<0>>),
    draw_glyph(Glyph).

bold_char_width(Font, C) ->
    Glyph = glyph_info(Font, C),
    glyph_width(Glyph)+1.

char(Font, C) ->
    draw_glyph(glyph_info(Font, C)).

draw_glyph({W,H,Xorig,Yorig,Xmove,B}) ->
    gl:bitmap(W, H, Xorig, Yorig, Xmove, 0, B).

glyph_info(#font{glyph_tab = GlyphTab, width = Width, height = Height, bitmaps = Bitmaps}, C) ->
    BitMap = case ets:lookup(GlyphTab, C) of
	[] when is_integer(C), C > 0 ->
	    %% Undefined character. Return a filled box.
	    NumBytes = ((Width+7) div 8) * Height,
	    B = <<(-1):NumBytes/unit:8>>,
	    {Width,Height,0,0,Width+1,B};
	[{_,W,H,Xorig,Yorig,Xmove,Offset}] ->
	    %% Valid character.
	    NumBytes = ((W+7) div 8)*H,
	    <<_:Offset/binary,B:NumBytes/binary,_/binary>> = Bitmaps,
	    {W,H,Xorig,Yorig,Xmove,B}
    end,
    insert_glyph(C, BitMap),
    BitMap.

glyph_width({_,_,_,_,Xmove,_}) -> Xmove.

insert_glyph(C, BitMap) ->
    Font = case wings_wm:this() of
        console -> console_font;
        _ -> system_font
    end,
    ets:insert(Font, {C,BitMap}).
